"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const spawn_async_1 = __importDefault(require("@expo/spawn-async"));
const chalk_1 = __importDefault(require("chalk"));
const globby_1 = __importDefault(require("globby"));
// @ts-ignore
const Runner_1 = __importDefault(require("jscodeshift/src/Runner"));
const multimatch_1 = __importDefault(require("multimatch"));
const path_1 = __importDefault(require("path"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const jscodeshift = require.resolve('jscodeshift/bin/jscodeshift.js');
const transformDir = path_1.default.join(__dirname, 'transforms');
const totalStats = {};
async function runAsync(argv) {
    const options = yargs_parser_1.default(argv);
    const [transform, ...paths] = options._.slice(2);
    const parser = options.parser;
    const transforms = await listTransformsAsync();
    if (options.help || !transform || paths.length === 0) {
        console.log(`
Usage: expo-codemod <transform> <paths...>

Options:
  <transform>                      (required) name of transform to apply to files
                                   (see a list of transforms available below)
  <paths...>                       one or more paths or globs (e.g. ./src/**/*.js) of sources to transform
                                   files in .gitignore are ignored by default
  -h, --help                       print this help message
  -p, --parser <babel|flow|ts|tsx> parser to use to parse the source files
                                   (default: babel, ts or tsx based on the file extension)

Transforms available:
  ${transforms.join('\n  ')}`);
        process.exit(1);
        return;
    }
    if (!transforms.includes(transform)) {
        throw new Error(`--transform: Transform ${transform} does not exist. Valid options: ` + transforms.join(', '));
    }
    const allFiles = await globby_1.default(paths, { gitignore: true, ignore: ['**/node_modules/**'] });
    if (parser) {
        console.log(`Transforming ${allFiles.length} files using parser '${parser}'...`);
        await transformAsync({ files: allFiles, parser, transform });
    }
    else {
        const jsFiles = multimatch_1.default(allFiles, ['**/*.js', '**/*.jsx']);
        if (jsFiles.length) {
            console.log(`Transforming ${jsFiles.length} JS files...`);
            await transformAsync({ files: jsFiles, parser: 'babel', transform });
        }
        const tsFiles = multimatch_1.default(allFiles, ['**/*.ts']);
        if (tsFiles.length) {
            console.log(`Transforming ${tsFiles.length} TS files...`);
            await transformAsync({ files: tsFiles, parser: 'ts', transform });
        }
        const tsxFiles = multimatch_1.default(allFiles, ['**/*.tsx']);
        if (tsxFiles.length) {
            console.log(`Transforming ${tsxFiles.length} TSX files...`);
            await transformAsync({ files: tsxFiles, parser: 'tsx', transform });
        }
    }
    if (transform === 'sdk33-imports' || transform === 'sdk37-imports') {
        const packages = Object.keys(totalStats).sort();
        if (packages.length) {
            console.log(chalk_1.default.bold(`\n🆕 Added imports from ${packages.length} packages. Installing packages with \`expo install\`...`));
            console.log(`> expo install ${packages.join(' ')}`);
            try {
                await spawn_async_1.default('expo', ['install', ...packages], { stdio: 'inherit' });
            }
            catch (error) {
                console.error(chalk_1.default.red('\n⚠️  expo install failed.'));
                console.log(chalk_1.default.dim(error.stack));
                console.error(chalk_1.default.red('To finish the codemod, please install the added packages manually using:'));
                console.log(chalk_1.default.bold.red(`> expo install ${packages.join(' ')}`));
                process.exit(1);
            }
        }
    }
}
exports.runAsync = runAsync;
async function transformAsync({ files, parser, transform, }) {
    const transformFile = path_1.default.join(transformDir, `${transform}.js`);
    if (transform === 'sdk33-imports' || transform === 'sdk37-imports') {
        // Dry run to gather stats about which packages need to be installed.
        const { stats } = await Runner_1.default.run(transformFile, files, {
            dry: true,
            silent: true,
            babel: false,
            parser,
        });
        Object.keys(stats).forEach(key => {
            totalStats[key] = (totalStats[key] || 0) + stats[key];
        });
    }
    const args = ['--parser', parser, '--transform', transformFile, '--no-babel', ...files];
    return await spawn_async_1.default(jscodeshift, args, { stdio: 'inherit' });
}
async function listTransformsAsync() {
    const modules = await globby_1.default(['*.js'], { cwd: transformDir });
    return modules.map(filename => path_1.default.basename(filename, '.js'));
}
//# sourceMappingURL=cli.js.map